/* crypto/pkcs7/sign.c */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <openca/general.h>
#include <openca/sign.h>
#include <openca/apps.h>

static char *usage[]={
"usage: sign argv\n",
"\n",
" -in file        - Input data file\n",
" -out file       - File where to write to (signature)\n",
" -cert file      - Certificate file\n",
" -passin arg     - Input file pass phrase source\n",
" -keyfile file   - Signing key file\n",
" -keyform arg    - Private key format (PEM or ENGINE)\n",
" -engine e       - Use engine e, possibly a hardware device\n",
" -othercerts file - Other certificates to be included\n",
" -key            - Password protecting the private key\n",
" -nd             - Non detached signature (data together with signature)\n",
" -no_sign        - Generate a PKCS#7 structure without signature\n",
" -verbose        - Verbose printing\n",
"\n",
NULL
};

int main(int argc, char *argv[])
{
	/* PARAMETERS CHECKING */
	int badops=0;
	int verbose=0;
	char *passargin = NULL;
	char *infile=NULL;
	char *outfile=NULL;
	char *certfile=NULL;
	char *othercertsfile=NULL;
	char *keyfile=NULL;
	char *key=NULL;
	int keyform=FORMAT_PEM;
	BIO *bio_err=NULL;
	BIO *Sout=NULL;
	char **pp = NULL ;

	/* PKCS7 USED VARIABLES */
	X509 *x509;
	EVP_PKEY *pkey = NULL ;
	const EVP_CIPHER *cipher = NULL;
	int flags = PKCS7_DETACHED;
	PKCS7 *p7 = NULL ;
	PKCS7_SIGNER_INFO *si = NULL ;
	BIO *certf = NULL , *keyf = NULL ;
	BIO *data = NULL ,*p7bio = NULL ;
	char buf[1024*4];
	int i;
	int nodetach=0;
	int is_signed=1;

	/* ADD ENGINE SUPPORT */
	char *engine=NULL;
	ENGINE *e = NULL;

	/* ADD ALL CHIPHERS */
	OpenSSL_add_all_ciphers();

	/* ADD DIGEST CYPHERS */
	OpenSSL_add_all_digests();

        if ((bio_err=BIO_new(BIO_s_file())) != NULL)
        	BIO_set_fp(bio_err,stderr,BIO_NOCLOSE|BIO_FP_TEXT);

	/* ARGUMENTS PARSING */
        argc--;
        argv++;
        while (argc >= 1)
                {
                if      (strcmp(*argv,"-in") == 0)
                        {
                        if (--argc < 1) goto bad;
                        infile= *(++argv);
                        }
		else if (strcmp(*argv,"-out") == 0)
                        {
                        if (--argc < 1) goto bad;
                        outfile= *(++argv);
                        }
		else if (strcmp(*argv,"-cert") == 0)
                        {
                        if (--argc < 1) goto bad;
                        certfile= *(++argv);
                        }
		else if (strcmp(*argv,"-othercerts") == 0)
                        {
                        if (--argc < 1) goto bad;
                        othercertsfile= *(++argv);
                        }
		else if (strcmp(*argv,"-keyfile") == 0)
                        {
                        if (--argc < 1) goto bad;
                        keyfile= *(++argv);
                        }
		else if (strcmp(*argv,"-keyform") == 0)
			{
			if (--argc < 1) goto bad;
			keyform=str2fmt(*(++argv));
			}
		else if (strcmp(*argv,"-engine") == 0)
			{
			if (--argc < 1) goto bad;
			engine= *(++argv);
			}
		else if (strcmp(*argv,"-key") == 0)
                        {
                        if (--argc < 1) goto bad;
                        key= *(++argv);
                        }
		else if (strcmp(*argv,"-passin") == 0)
			{
			if (--argc < 1) goto bad;
			passargin= *(++argv);
			}
                else if (strcmp(*argv,"-no_sign") == 0)
                        is_signed=0;
                else if (strcmp(*argv,"-verbose") == 0)
                        verbose=1;
                else if (strcmp(*argv,"-nd") == 0)
                        nodetach=1;
#ifndef OPENSSL_NO_DES
		else if (!strcmp (*argv, "-des3")) 
				cipher = EVP_des_ede3_cbc();
		else if (!strcmp (*argv, "-des")) 
				cipher = EVP_des_cbc();
#endif
#ifndef OPENSSL_NO_RC2
		else if (!strcmp (*argv, "-rc2-40")) 
				cipher = EVP_rc2_40_cbc();
		else if (!strcmp (*argv, "-rc2-128")) 
				cipher = EVP_rc2_cbc();
		else if (!strcmp (*argv, "-rc2-64")) 
				cipher = EVP_rc2_64_cbc();
#endif
#ifndef OPENSSL_NO_AES
		else if (!strcmp(*argv,"-aes128"))
				cipher = EVP_aes_128_cbc();
		else if (!strcmp(*argv,"-aes192"))
				cipher = EVP_aes_192_cbc();
		else if (!strcmp(*argv,"-aes256"))
				cipher = EVP_aes_256_cbc();
#endif
		else if (!strcmp (*argv, "-text")) 
				flags |= PKCS7_TEXT;
		else if (!strcmp (*argv, "-nointern")) 
				flags |= PKCS7_NOINTERN;
		else if (!strcmp (*argv, "-noverify")) 
				flags |= PKCS7_NOVERIFY;
		else if (!strcmp (*argv, "-nochain")) 
				flags |= PKCS7_NOCHAIN;
		else if (!strcmp (*argv, "-nocerts")) 
				flags |= PKCS7_NOCERTS;
		else if (!strcmp (*argv, "-noattr")) 
				flags |= PKCS7_NOATTR;
		else if (!strcmp (*argv, "-nodetach")) 
				flags &= ~PKCS7_DETACHED;
		else if (!strcmp (*argv, "-nosmimecap"))
				flags |= PKCS7_NOSMIMECAP;
		else if (!strcmp (*argv, "-binary"))
				flags |= PKCS7_BINARY;
		else if (!strcmp (*argv, "-nosigs"))
				flags |= PKCS7_NOSIGS;
		else
                        {
bad:
                        BIO_printf(bio_err,"unknown option %s\n",*argv);
                        badops=1;
                        break;
                        }
                argc--;
                argv++;
                }

	if( !keyfile && is_signed )
		badops=1;

	if( (!certfile) || (!infile && is_signed) ) 
		badops=1;

	if (badops)
                {
                for (pp=usage; (*pp != NULL); pp++)
                        BIO_printf(bio_err,*pp);
                	exit(1);
                }

	if (!generatePKCS7(OPENCA_PKCS7_SIGN, verbose, infile, outfile, certfile, keyfile,
                                  key, keyform, passargin, nodetach, engine, is_signed, othercertsfile,
	                          cipher, flags ))
		goto err;

	exit(0);

err:
	ERR_print_errors_fp(stderr);
	exit(1);
	}
